# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "剩余 %d 分钟去阅读",
    "(active)": "(活跃)",
    "Also available in:": "也可用于：",
    "Archive": "文章归档",
    "Atom feed": "Atom源",
    "Authors": "作者",
    "Categories": "分类",
    "Comments": "评论",
    "LANGUAGE": "简体中文",
    "Languages:": "语言：",
    "More posts about %s": "更多关于文章 %s",
    "Newer posts": "较新的文章",
    "Next post": "下一篇文章",
    "Next": "项下",
    "No posts found.": "没有找到文章",
    "Nothing found.": "没有找到。",
    "Older posts": "以前的文章",
    "Original site": "原文地址",
    "Posted:": "发表于：",
    "Posts about %s": "关于文章 %s",
    "Posts by %s": "文章有%s发布",
    "Posts for year %s": "%s年文章",
    "Posts for {month_day_year}": "{month_day_year}文章",
    "Posts for {month_year}": "{month_year}文章",
    "Previous post": "上一篇文章",
    "Previous": "以前",
    "Publication date": "发布日期",
    "RSS feed": "RSS 源",
    "Read in English": "阅读简体中文",
    "Read more": "阅读更多",
    "Skip to main content": "跳到主内容",
    "Source": "资源",
    "Subcategories:": "子类别：",
    "Tags and Categories": "标签和分类",
    "Tags": "标签",
    "Toggle navigation": "展开导航栏",
    "Uncategorized": "未分类",
    "Up": "向上",
    "Updates": "更新",
    "Write your page here.": "在这里书写你的页面。",
    "Write your post here.": "在这里书写你的文章。",
    "old posts, page %d": "旧文章页 %d",
    "page %d": "第 %d 页",
    "updated": "更新",
}
