# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "पढ़ने में %d मिनट बाकी",
    "(active)": "(सक्रिय)",
    "Also available in:": "उपलब्ध भाषाएँ:",
    "Archive": "आर्काइव",
    "Atom feed": "",
    "Authors": "लेखक",
    "Categories": "श्रेणियाँ",
    "Comments": "टिप्पणियाँ",
    "LANGUAGE": "हिन्दी",
    "Languages:": "भाषाएँ:",
    "More posts about %s": "%s के बारे में अौर पोस्टें",
    "Newer posts": "नई पोस्टें",
    "Next post": "अगली पोस्ट",
    "Next": "",
    "No posts found.": "कोई पोस्ट नहीं मिल सकी",
    "Nothing found.": "कुछ नहीं मिल सका",
    "Older posts": "पुरानी पोस्टें",
    "Original site": "असली साइट",
    "Posted:": "पोस्टेड:",
    "Posts about %s": "%s के बारे में पोस्टें",
    "Posts by %s": "%s  की पोस्टें",
    "Posts for year %s": "साल %s की पोस्टें",
    "Posts for {month_day_year}": "{month_day_year} की पोस्टें",
    "Posts for {month_year}": "{month_year} की पोस्टें",
    "Previous post": "पिछली पोस्ट",
    "Previous": "",
    "Publication date": "प्रकाशन की तारीख",
    "RSS feed": "आर एस एस फ़ीड",
    "Read in English": "हिन्दी में पढ़िए",
    "Read more": "और पढ़िए",
    "Skip to main content": "मुख्य सामग्री पर जाएँ",
    "Source": "सोर्स",
    "Subcategories:": "उपश्रेणी",
    "Tags and Categories": "टैग्स और श्रेणियाँ",
    "Tags": "टैग्स",
    "Toggle navigation": "",
    "Uncategorized": "बिना श्रेणी",
    "Up": "",
    "Updates": "अपडेट्स",
    "Write your page here.": "अपना पेज यहाँ लिखिए",
    "Write your post here.": "अपनी पोस्ट यहाँ लिखिए",
    "old posts, page %d": "पुरानी पोस्टें, पृष्‍ठ %d",
    "page %d": "पृष्‍ठ %d",
    "updated": "",
}
