# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "残りを読むのに必要な時間は%d分",
    "(active)": "(有効)",
    "Also available in:": "他の言語で読む:",
    "Archive": "文書一覧",
    "Atom feed": "Atomフィード",
    "Authors": "著者一覧",
    "Categories": "カテゴリ",
    "Comments": "コメント",
    "LANGUAGE": "日本語",
    "Languages:": "言語:",
    "More posts about %s": "%sに関する文書一覧",
    "Newer posts": "新しい文書",
    "Next post": "次の文書",
    "Next": "次",
    "No posts found.": "文書はありません。",
    "Nothing found.": "なにも見つかりませんでした。",
    "Older posts": "過去の文書",
    "Original site": "翻訳元のサイト",
    "Posted:": "公開日時:",
    "Posts about %s": "%sについての文書",
    "Posts by %s": "%sの文書一覧",
    "Posts for year %s": "%s年の文書",
    "Posts for {month_day_year}": "{month_day_year}の文書",
    "Posts for {month_year}": "{month_year}の文書",
    "Previous post": "一つ前の文書",
    "Previous": "前",
    "Publication date": "公開日",
    "RSS feed": "RSSフィード",
    "Read in English": "日本語で読む",
    "Read more": "続きを読む",
    "Skip to main content": "本文を読み飛ばす",
    "Source": "ソース",
    "Subcategories:": "サブカテゴリ",
    "Tags and Categories": "カテゴリおよびタグ一覧",
    "Tags": "タグ",
    "Toggle navigation": "ナビゲーションを隠す",
    "Uncategorized": "uncategorized",
    "Up": "上",
    "Updates": "フィード",
    "Write your page here.": "ここに文書を記述してください。",
    "Write your post here.": "ここに文書を記述してください。",
    "old posts, page %d": "過去の文書 %dページ目",
    "page %d": "ページ%d",
    "updated": "",
}
