# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "%d min ngelen për tu lexuar",
    "(active)": "(aktiv)",
    "Also available in:": "Gjithashtu e disponueshme në:",
    "Archive": "Arkiva",
    "Atom feed": "",
    "Authors": "Autorë",
    "Categories": "Kategori",
    "Comments": "Komente",
    "LANGUAGE": "Shqip",
    "Languages:": "Gjuhë:",
    "More posts about %s": "Më shumë postime rreth %s",
    "Newer posts": "Postime më të reja",
    "Next post": "Postimi i rradhës",
    "Next": "",
    "No posts found.": "Nuk është gjetur asnjë post.",
    "Nothing found.": "Nuk është gjetur asgjë.",
    "Older posts": "Postime më të vjetra",
    "Original site": "Faqja origjinale",
    "Posted:": "Postuar:",
    "Posts about %s": "Postime rreth %s",
    "Posts by %s": "Postime nga %s",
    "Posts for year %s": "Postime për vitin %s",
    "Posts for {month_day_year}": "Postime për {month_day_year}",
    "Posts for {month_year}": "Postime për {month_year}",
    "Previous post": "Postim i kaluar",
    "Previous": "",
    "Publication date": "Data e publikimit",
    "RSS feed": "Furnizim RSS",
    "Read in English": "Lexo në Shqip",
    "Read more": "Lexo më shumë",
    "Skip to main content": "Shko tek përmbajtja kryesore",
    "Source": "Burim",
    "Subcategories:": "Nënkategori:",
    "Tags and Categories": "Etiketa dhe Kategori",
    "Tags": "Etiketa",
    "Toggle navigation": "",
    "Uncategorized": "E pa kategorizuar",
    "Up": "",
    "Updates": "Përditësime",
    "Write your page here.": "Shkruaj faqen tënde këtu.",
    "Write your post here.": "Shkruaj postin tënd këtu.",
    "old posts, page %d": "Postime të kaluara, faqe %d",
    "page %d": "faqe %d",
    "updated": "",
}
