# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "%d minutu gelditzen dira irakurtzeko",
    "(active)": "(aktibo)",
    "Also available in:": "Eskuragarria hizkuntza hauetan ere:",
    "Archive": "Artxiboa",
    "Atom feed": "",
    "Authors": "Egileak",
    "Categories": "Kategoriak",
    "Comments": "Iruzkinak",
    "LANGUAGE": "Euskara",
    "Languages:": "Hizkuntzak:",
    "More posts about %s": "%s-ri buruzko argitalpen gehiago",
    "Newer posts": "Argitalpen berriagoak",
    "Next post": "Hurrengo argitalpena",
    "Next": "",
    "No posts found.": "Ez da argitalpenik aurkitu",
    "Nothing found.": "Ez da ezer aurkitu",
    "Older posts": "Argitalpen zaharragoak",
    "Original site": "Jatorrizko gunea",
    "Posted:": "Argitaratuta:",
    "Posts about %s": "%s-ri buruzko argitalpenak",
    "Posts by %s": "%s-ek idatzitako argitalpenak",
    "Posts for year %s": "%s. urteko argitalpenak",
    "Posts for {month_day_year}": "{month_day_year} argitalpenak",
    "Posts for {month_year}": "{month_year} argitalpenak",
    "Previous post": "Aurreko argitalpena",
    "Previous": "",
    "Publication date": "Argitaratze-data",
    "RSS feed": "RSS jarioa",
    "Read in English": "Euskaraz irakurri",
    "Read more": "Irakurri gehiago",
    "Skip to main content": "Joan eduki nagusira",
    "Source": "Iturria",
    "Subcategories:": "Azpikategoriak:",
    "Tags and Categories": "Etiketak eta Kategoriak",
    "Tags": "Etiketak",
    "Toggle navigation": "",
    "Uncategorized": "Kategorizatu-gabeak",
    "Up": "",
    "Updates": "Eguneraketak",
    "Write your page here.": "Idatzi zure orria hemen",
    "Write your post here.": "Idatzi zure argitalpena hemen",
    "old posts, page %d": "Argitalpen zaharragoak, %d. orria",
    "page %d": "%d. orria",
    "updated": "",
}
