# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "%d دقائق متبقية للقراءة",
    "(active)": "(نشط)",
    "Also available in:": "أيضا متوفر في:",
    "Archive": "الأرشيف",
    "Atom feed": "",
    "Authors": "المؤلفون",
    "Categories": "فئات",
    "Comments": "التّعليقات",
    "LANGUAGE": "العربيّة",
    "Languages:": "اللغات",
    "More posts about %s": "المزيد من المقالات حول %s",
    "Newer posts": "مقالات أحدث",
    "Next post": "المقالة التالية",
    "Next": "التالي",
    "No posts found.": "لم يوجد مقالات.",
    "Nothing found.": "لم يوجد شيء.",
    "Older posts": "مقالات أقدم",
    "Original site": "الموقع الأصلي",
    "Posted:": "نشر:",
    "Posts about %s": "مقالات عن %s",
    "Posts by %s": "مقالات بواسطة %s",
    "Posts for year %s": "مقالات سنة %s",
    "Posts for {month_day_year}": "",
    "Posts for {month_year}": "",
    "Previous post": "المقالة السابقة",
    "Previous": "السابق",
    "Publication date": "تاريخ النشر",
    "RSS feed": "",
    "Read in English": "اقرأ بالعربية",
    "Read more": "قراءة المزيد",
    "Skip to main content": "انتقل إلى المحتوى الرئيسي",
    "Source": "المصدر",
    "Subcategories:": "فئات فرعية",
    "Tags and Categories": "تصنيفات و فئات",
    "Tags": "تصنيفات",
    "Toggle navigation": "",
    "Uncategorized": "غير مصنف",
    "Up": "أعلى",
    "Updates": "التحديثات",
    "Write your page here.": "أكتب صفحتك هنا ",
    "Write your post here.": "أكتب مقالك هنا",
    "old posts, page %d": "مقالات قديمة, صفحة %d",
    "page %d": "صفحة %d",
    "updated": "",
}
