# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "%d דקות נותרים לסיום קריאה",
    "(active)": "(פעיל)",
    "Also available in:": "זמין גם ב:",
    "Archive": "ארכיב",
    "Atom feed": "",
    "Authors": "מחברים",
    "Categories": "קטגוריות",
    "Comments": "הערות",
    "LANGUAGE": "אנגלית",
    "Languages:": "שפות:",
    "More posts about %s": "עוד פוסטים אודות %s",
    "Newer posts": "פוסטים חדשים",
    "Next post": "לפוסט הבא",
    "Next": "",
    "No posts found.": "לא נמצאו פוסטים",
    "Nothing found.": "לא נמצא",
    "Older posts": "פוסטים ישנים",
    "Original site": "אתר המקורי",
    "Posted:": "פורסם:",
    "Posts about %s": "פוסטים אודות %s",
    "Posts by %s": "פוסטים ע״י %s",
    "Posts for year %s": "פוסטים לשנת %s",
    "Posts for {month_day_year}": "פוסטים עבוד {month_day_year}",
    "Posts for {month_year}": "פוסטים עבוד {month_year}",
    "Previous post": "פוסט הקודם",
    "Previous": "",
    "Publication date": "תאריך פרסום",
    "RSS feed": "פיד RSS",
    "Read in English": "קרא באנגלית",
    "Read more": "קרא עוד",
    "Skip to main content": "דלג לתוכן ראשי",
    "Source": "מקור",
    "Subcategories:": "תתי קטגוריות:",
    "Tags and Categories": "תגים וקטגוריות",
    "Tags": "תגים",
    "Toggle navigation": "החלף מצב ניווט",
    "Uncategorized": "לא משויך לקטגוריה",
    "Up": "",
    "Updates": "עדכונים",
    "Write your page here.": "תכתוב את העמוד שלך פה.",
    "Write your post here.": "תכתוב את הפוסט שלך פה.",
    "old posts, page %d": "פוסטים קודמים, דף %d",
    "page %d": "עמוד %d",
    "updated": "",
}
