# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "ใช้เวลาอ่านอีก %d นาที",
    "(active)": "(ถูกใช้งาน)",
    "Also available in:": "ภาษาอื่นๆ:",
    "Archive": "คลังโพสต์",
    "Atom feed": "ฟีด Atom",
    "Authors": "ผู้เขียน",
    "Categories": "หมวดหมู่",
    "Comments": "ความคิดเห็น",
    "LANGUAGE": "ภาษาไทย",
    "Languages:": "ภาษา:",
    "More posts about %s": "โพสต์เพิ่มเติมเกี่ยวกับ %s",
    "Newer posts": "โพสต์มาใหม่",
    "Next post": "โพสต์ถัดไป",
    "Next": "ถัดไป",
    "No posts found.": "ไม่พบโพสต์.",
    "Nothing found.": "ไม่พบ",
    "Older posts": "โพสต์เก่า",
    "Original site": "เว็บไซต์ที่มา",
    "Posted:": "โพสต์เมื่อ:",
    "Posts about %s": "โพสต์เกี่ยวกับ %s",
    "Posts by %s": "โพสต์โดย %s",
    "Posts for year %s": "โพสต์เมื่อปี  %s",
    "Posts for {month_day_year}": "โพสต์เมื่อ {month_day_year}",
    "Posts for {month_year}": "โพสต์เมื่อ  {month_year}",
    "Previous post": "โพสต์ก่อนหน้า",
    "Previous": "ก่อนหน้า",
    "Publication date": "วันที่ตีพิมพ์",
    "RSS feed": "ฟีด RSS",
    "Read in English": "อ่านเป็นภาษาไทย",
    "Read more": "อ่านเพิ่มเติม",
    "Skip to main content": "ข้ามไปหน้าหลัก",
    "Source": "แหล่งที่มา",
    "Subcategories:": "หมวดหมู่ย่อย:",
    "Tags and Categories": "แท็กและหมวดหมู่",
    "Tags": "แท็ก",
    "Toggle navigation": "เมนูบาร์",
    "Uncategorized": "ไม่มีหมวดหมู่",
    "Up": "ขึ้นด้านบน",
    "Updates": "อัพเดด",
    "Write your page here.": "เขียนเพจที่นี่",
    "Write your post here.": "เขียนโพสต์ที่นี่",
    "old posts, page %d": "โพสต์เก่า, หน้า %d",
    "page %d": "หน้า %d",
    "updated": "",
}
