# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "%d minutoj por legi",
    "(active)": "(aktiva)",
    "Also available in:": "Ankaŭ disponebla en:",
    "Archive": "Arkivo",
    "Atom feed": "Atom fluo",
    "Authors": "Aŭtoroj",
    "Categories": "Kategorioj",
    "Comments": "Komentoj",
    "LANGUAGE": "Esperante",
    "Languages:": "Lingvoj:",
    "More posts about %s": "Pli da artikoloj pri %s",
    "Newer posts": "Pli novaj artikoloj",
    "Next post": "Venonta artikolo",
    "Next": "Venonta",
    "No posts found.": "Neniu artikoloj trovitaj.",
    "Nothing found.": "Nenio trovita.",
    "Older posts": "Pli malnovaj artikoloj",
    "Original site": "Originala retejo",
    "Posted:": "Skribita:",
    "Posts about %s": "Artikoloj pri %s",
    "Posts by %s": "Artikoloj de %s",
    "Posts for year %s": "Artikoloj de la jaro %s",
    "Posts for {month_day_year}": "Artikoloj de la {month_day_year}",
    "Posts for {month_year}": "Artikoloj de {month_year}",
    "Previous post": "Antaŭa artikolo",
    "Previous": "Antaŭa",
    "Publication date": "Eldona dato",
    "RSS feed": "RSS fluo",
    "Read in English": "Legu ĝin en Esperanto",
    "Read more": "Legu pli",
    "Skip to main content": "Salti al ĉefenhavo",
    "Source": "Fonto",
    "Subcategories:": "Subkategorioj:",
    "Tags and Categories": "Etikedoj kaj kategorioj",
    "Tags": "Etikedoj",
    "Toggle navigation": "Ŝalti menuon",
    "Uncategorized": "Sen kategorioj",
    "Up": "Supren",
    "Updates": "Ĝisdatigoj",
    "Write your page here.": "Skribu tie vian paĝon.",
    "Write your post here.": "Skribu tie vian artikolon.",
    "old posts, page %d": "%da paĝo de malnovaj artikoloj",
    "page %d": "paĝo %d",
    "updated": "",
}
