# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "%d menit tersisa untuk membaca",
    "(active)": "(aktif)",
    "Also available in:": "Juga tersedia dalam:",
    "Archive": "Arsip",
    "Atom feed": "Umpan Atom",
    "Authors": "Penulis",
    "Categories": "Kategori",
    "Comments": "Komentar",
    "LANGUAGE": "Bahasa Indonesia",
    "Languages:": "Bahasa:",
    "More posts about %s": "Lebih banyak tulisan tentang %s",
    "Newer posts": "Tulisan lebih baru",
    "Next post": "Tulisan berikutnya",
    "Next": "Sesudah",
    "No posts found.": "Tidak ada tulisan yang ditemukan.",
    "Nothing found.": "Tidak ditemukan.",
    "Older posts": "Tulisan lebih lama",
    "Original site": "Situs orisinal",
    "Posted:": "Ditulis oleh:",
    "Posts about %s": "Tulisan tentang %s",
    "Posts by %s": "Tulisan oleh %s",
    "Posts for year %s": "Tulisan untuk tahun %s",
    "Posts for {month_day_year}": "Tulisan untuk {month_day_year}",
    "Posts for {month_year}": "Tulisan untuk {month_year}",
    "Previous post": "Tulisan sebelumnya",
    "Previous": "Sebelum",
    "Publication date": "Tanggal publikasi",
    "RSS feed": "Sindikasi RSS",
    "Read in English": "Baca dalam Bahasa Indonesia",
    "Read more": "Baca selengkapnya",
    "Skip to main content": "Lanjutkan ke konten utama",
    "Source": "Sumber",
    "Subcategories:": "Sub kategori:",
    "Tags and Categories": "Tag dan Kategori",
    "Tags": "Tag",
    "Toggle navigation": "Alih navigasi",
    "Uncategorized": "Tanpa kategori",
    "Up": "Atas",
    "Updates": "Update",
    "Write your page here.": "Tulis halaman Anda disini.",
    "Write your post here.": "Tulis tulisan Anda disini.",
    "old posts, page %d": "tulisan lama, halaman %d",
    "page %d": "halaman %d",
    "updated": "",
}
