# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "liko %d min skaitymo",
    "(active)": "(aktyvi)",
    "Also available in:": "Taip pat turimas šiomis kalbomis:",
    "Archive": "Archyvas",
    "Atom feed": "",
    "Authors": "Autoriai",
    "Categories": "Kategorijos",
    "Comments": "Komentarai",
    "LANGUAGE": "Lietuvių",
    "Languages:": "Kalbos:",
    "More posts about %s": "Daugiau įrašų apie %s",
    "Newer posts": "Naujesni įrašai",
    "Next post": "Sekantis įrašas",
    "Next": "",
    "No posts found.": "Įrašų nerasta.",
    "Nothing found.": "Nieko nerasta.",
    "Older posts": "Senesni įrašai",
    "Original site": "Originalo svetainė",
    "Posted:": "Įrašyta:",
    "Posts about %s": "Įrašai apie %s",
    "Posts by %s": "Autoriaus %s įrašai",
    "Posts for year %s": "%s metų įrašai",
    "Posts for {month_day_year}": "{month_day_year} įrašai",
    "Posts for {month_year}": "{month_year} įrašai",
    "Previous post": "Ankstesnis įrašas",
    "Previous": "",
    "Publication date": "Publikavimo data",
    "RSS feed": "RSS srautas",
    "Read in English": "Skaityti lietuviškai",
    "Read more": "Skaityti daugiau",
    "Skip to main content": "Pereiti į pagrindinį turinį",
    "Source": "Pirminis tekstas",
    "Subcategories:": "Subkategorijos:",
    "Tags and Categories": "Žymės ir Kategorijos",
    "Tags": "Žymės",
    "Toggle navigation": "",
    "Uncategorized": "Nekategorizuota",
    "Up": "",
    "Updates": "Atnaujinimai",
    "Write your page here.": "Čia rašykite puslapio tekstą.",
    "Write your post here.": "Čia rašykite įrašo tekstą.",
    "old posts, page %d": "seni įrašai, %d puslapis",
    "page %d": "%d puslapis",
    "updated": "",
}
