# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "%d മിനിട്ട് കൂടി വായിച്ചു തീരാന്‍  ",
    "(active)": "(സജീവം)",
    "Also available in:": "ലഭ്യമായ ഭാഷകള്‍:",
    "Archive": "ആര്‍കൈവ്",
    "Atom feed": "ആറ്റം ഫിഡ്",
    "Authors": "രചയിതാക്കള്‍",
    "Categories": "വിഭാഗങ്ങള്‍",
    "Comments": "കമന്റുകള്‍",
    "LANGUAGE": "മലയാളം",
    "Languages:": "ഭാഷകള്‍:",
    "More posts about %s": "%s-നെ കുറിച്ചുള്ള കൂടുതല്‍ രചനകള്‍",
    "Newer posts": "പുതിയ രചനകള്‍",
    "Next post": "അടുത്ത രചന",
    "Next": "അടുത്തത്",
    "No posts found.": "രചനകള്‍ ലഭ്യമല്ല.",
    "Nothing found.": "ഒന്നും ലഭ്യമല്ല.",
    "Older posts": "പഴയ രചനകള്‍",
    "Original site": "യഥാര്‍ഥ സൈറ്റ്",
    "Posted:": "എഴുതിയത്:",
    "Posts about %s": "%s -നെ കുറിച്ചുള്ള രചനകള്‍",
    "Posts by %s": "%s എഴുതിയ രചനകള്‍",
    "Posts for year %s": "%s-ാം ആണ്ടിലെ രചനകള്‍",
    "Posts for {month_day_year}": "{month_day_year} ലെ രചനകള്‍",
    "Posts for {month_year}": "{month_year} ലെ രചനകള്‍",
    "Previous post": "മുമ്പിലെ രചന",
    "Previous": "കഴിഞ്ഞത്",
    "Publication date": "പ്രസിദ്ധീകരിച്ച തീയതി",
    "RSS feed": "ആര്‍ എസ് എസ് ഫീഡ്",
    "Read in English": "മലയാളത്തില്‍ വായിക്കുക",
    "Read more": "കൂടുതല്‍ വായിക്കുക",
    "Skip to main content": "പ്രധാന ഉള്ളടക്കത്തിലേക്ക് നേരെ പോവുക",
    "Source": "സ്രോതസ്സ്‌",
    "Subcategories:": "ഉപവിഭാഗങ്ങള്‍:",
    "Tags and Categories": "ടാഗുകളും വിഭാഗങ്ങളും",
    "Tags": "ടാഗുകള്‍",
    "Toggle navigation": "നാവിഗേഷൻ മാറ്റുക",
    "Uncategorized": "വേര്‍തിരിക്കാത്തവ",
    "Up": "മുകളിലേക്ക്",
    "Updates": "അപ്ഡേറ്റുകൾ",
    "Write your page here.": "താങ്കളുടെ താള്‍ ഇവിടെ എഴുതുക.",
    "Write your post here.": "താങ്കളുടെ രചന ഇവിടെ എഴുതുക.",
    "old posts, page %d": "പഴയ രചനകള്‍, താള്‍ %d",
    "page %d": "താള്‍ %d",
    "updated": "പുതുക്കിയത്",
}
