# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "%d minutit veel lugeda",
    "(active)": "(aktiivne)",
    "Also available in:": "Saadaval ka:",
    "Archive": "Arhiiv",
    "Atom feed": "Atom uudisvoog",
    "Authors": "Autorid",
    "Categories": "Kategooriad",
    "Comments": "Kommentaarid",
    "LANGUAGE": "Eesti",
    "Languages:": "Keeled:",
    "More posts about %s": "Veel postitusi %s kohta",
    "Newer posts": "Uued postitused",
    "Next post": "Järgmine postitus",
    "Next": "Järgmine",
    "No posts found.": "Postitusi ei leitud.",
    "Nothing found.": "Midagi ei leitud.",
    "Older posts": "Vanemad postitused",
    "Original site": "Algallikas",
    "Posted:": "Postitatud:",
    "Posts about %s": "Postitused %s kohta",
    "Posts by %s": "Postitus kasutajalt %s",
    "Posts for year %s": "Postitused aastast %s",
    "Posts for {month_day_year}": "Postitused {month_day_year}",
    "Posts for {month_year}": "Postitused {month_year}",
    "Previous post": "Eelmine postitus",
    "Previous": "Eelmine",
    "Publication date": "Avaldamise kuupäev",
    "RSS feed": "RSS uudisvoog",
    "Read in English": "Loe eesti keeles",
    "Read more": "Loe veel",
    "Skip to main content": "Otse peamise sisu juurde",
    "Source": "Lähtekood",
    "Subcategories:": "Alamkategooriad:",
    "Tags and Categories": "Sildid ja kategooriad",
    "Tags": "Märksõnad",
    "Toggle navigation": "Näita või peida menüüd",
    "Uncategorized": "Kategooriata",
    "Up": "Üles",
    "Updates": "Uuendused",
    "Write your page here.": "Kirjuta oma lehekülje sisu siia.",
    "Write your post here.": "Kirjuta oma postitus siia.",
    "old posts, page %d": "vanade postituste, leht %d",
    "page %d": "leht %d",
    "updated": "",
}
