# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "%d منٹ کا مطالعہ باقی",
    "(active)": "(فعال)",
    "Also available in:": "ان میں بھی دستیاب:",
    "Archive": "آرکائیو",
    "Atom feed": "ایٹم فِیڈ",
    "Authors": "مصنفین",
    "Categories": "زمرے",
    "Comments": "تبصرے",
    "LANGUAGE": "اردو",
    "Languages:": "زبانیں:",
    "More posts about %s": "%s کے بارے میں مزید تحاریر",
    "Newer posts": "نئی تحاریر",
    "Next post": "اگلی تحریر",
    "Next": "آگے",
    "No posts found.": "کوئی تحریر نہیں مل سکی۔",
    "Nothing found.": "کچھ نہیں مل سکا۔",
    "Older posts": "پرانی تحاریر",
    "Original site": "اصلی سائٹ",
    "Posted:": "اشاعت:",
    "Posts about %s": "%s کے بارے میں تحاریر",
    "Posts by %s": "%s کی تحاریر",
    "Posts for year %s": "سال %s کی تحاریر",
    "Posts for {month_day_year}": "{month_day_year} کی تحاریر",
    "Posts for {month_year}": "{month_year} کی تحاریر",
    "Previous post": "پچھلی تحریر",
    "Previous": "پیچھے",
    "Publication date": "تاریخِ اشاعت",
    "RSS feed": "آر ایس ایس فیڈ",
    "Read in English": "اردو میں پڑھیں",
    "Read more": "مزید پڑھیں",
    "Skip to main content": "مرکزی متن پر جائیں",
    "Source": "سورس",
    "Subcategories:": "ذیلی زمرے",
    "Tags and Categories": "ٹیگز اور زمرے",
    "Tags": "ٹیگز",
    "Toggle navigation": "نیویگیشن ہٹائیں/دکھائیں",
    "Uncategorized": "بے زمرہ",
    "Up": "اوپر",
    "Updates": "تازہ ترین",
    "Write your page here.": "اپنے صفحے کا متن یہاں لکھیں۔",
    "Write your post here.": "اپنی تحریر یہاں لکھیں۔",
    "old posts, page %d": "پرانی تحاریر صفحہ %d",
    "page %d": "صفحہ %d",
    "updated": "",
}
